part of '../bolter_provider.dart';

class Presenter<FS> with Loading {
  Bolter _bolter;
  FS _flutterState;
  UseCaseContainer _useCaseContainer;
  BuildContext _context;

  FS get flutterState => _flutterState;

  @protected
  ValueStream<V> stream<V>(Getter<V> getter, {bool distinct = true}) =>
      _bolter.stream(getter, distinct: distinct);

  @protected
  FutureOr<void> init() {}

  @protected
  void updateUI() => _bolter.shake();

  @protected
  U useCase<U>() => _useCaseContainer.useCase<U>();

  UseCaseContainer get container => _useCaseContainer;

  @protected
  void runContext(void Function(BuildContext context) handle) {
    if (_context != null) {
      handle(_context);
    }
  }

  @override
  @protected
  @mustCallSuper
  void dispose() {
    _context = null;
    super.dispose();
  }
}

mixin NavigationPresenter<FS> on Presenter<FS> {
  ValueStream<List<BolterRoute>> get routesStream =>
      _bolter.stream(() => navigation.routes);

  Navigation get navigation;

  void pop() {
    navigation.pop();
    _bolter.shake();
  }

  void popUntil<T extends BolterRoute>() {
    navigation.popUntil<T>();
    _bolter.shake();
  }

  Future<R> push<R>(BolterRoute<R> route) {
    navigation.push(route);
    _bolter.shake();
    return route.whenComplete;
  }

  Future<R> pushReplacement<R>(BolterRoute<R> route) {
    navigation.pushReplacement(route);
    _bolter.shake();
    return route.whenComplete;
  }

  Future<R> pushAndRemoveUntil<R>(BolterRoute<R> route) {
    navigation.pushAndRemoveUntil(route);
    _bolter.shake();
    return route.whenComplete;
  }

  List<BolterRoute<Object>> get currentRoutes => navigation.routes;

  BolterRoute<Object> get currentRoute => currentRoutes.last;

  T findRoute<T extends BolterRoute>() => navigation.findRoute<T>();

  BolterRoute<Object> get firstRoute => currentRoutes.first;

  bool isInStack<T extends BolterRoute>() =>
      currentRoutes.firstWhere((element) => element is T, orElse: () => null) !=
      null;

  bool get canPop => navigation.routes.length > 1;

  BolterRoute<Object> get routeBefore => currentRoutes.length > 1
      ? currentRoutes[currentRoutes.length - 2]
      : throw null;
}

mixin TabNavigationPresenter<FS> on Presenter<FS> {
  TabNavigation get tabNavigation;

  ValueStream<String> get currentTabStream =>
      _bolter.stream(() => tabNavigation.tab);

  String get currentTab => tabNavigation.tab;

  void changeTab(String newTab) {
    tabNavigation.tab = newTab;
    _bolter.shake();
  }
}
